# load libraries
library(rags2ridges)

# load estimated precision matrices
load("ridgePcancer.Rdata")
load("ridgePmetastatis.Rdata")

# load data
load("lungTCGAdata.Rdata")
load("lungTCGAdata_withNames.Rdata")
p      <- nrow(GE0)
gNames <- rownames(GEdata)

# construct optimal penalty matrix
Y1 <- cbind(t(GE1), t(CN1))
S  <- t(Y1) %*% Y1 / nrow(Y1)

# add names to rows and columns
colnames(Pcnt) <- rownames(Pcnt) <- c(gNames, paste(gNames, "cn", sep=""))
colnames(Pmwt) <- rownames(Pmwt) <- c(gNames, paste(gNames, "cn", sep=""))

# heatmaps of estimated partial correlation matrices
edgeHeat(pcor(Pcnt), diag=FALSE, 
	 main="estimated partial correlation matrix, cancer, null target")
edgeHeat(pcor(Pmwt), diag=FALSE, 
         main="estimated partial correlation matrix, metastasis, informative target")

# heatmaps of estimated partial correlation matrices (among expression levels only)
edgeHeat(pcor(Pcnt)[1:p, 1:p], diag=FALSE, 
	main="est. partial correlation matrix, cancer, null target, g.e. only")
edgeHeat(pcor(Pmwt)[1:p, 1:p], diag=FALSE, 
         main="est. partial correlation matrix, metastasis, informative target, g.e. only")

# histogram of the copynumber-gene expression partial correlations
hist(diag(pcor(Pcnt)[1:p, c((p+1):(2*p))]), col="blue", border="lightblue", n=25, 
     xlab="copy number-gene expression partial correlation", 
     main="Cancer, non-informative target")
hist(diag(pcor(Pmwt)[1:p, c((p+1):(2*p))]), col="blue", border="lightblue", n=25, 
     xlab="copy number-gene expression partial correlation", 
     main="Metastasis, informative target")

# scatterplot of the gene expression partial correlations (cancer vs metastasis)
plot(x=pcor(Pcnt)[1:p, 1:p][upper.tri(diag(p))], 
     y=pcor(Pmwt)[1:p, 1:p][upper.tri(diag(p))], 
     pch=20, xlab=c("cancer"), ylab=("metastasis"))

# scatterplot of the copynumber-gene expression partial correlations (cancer vs metastasis)
plot(x=diag(pcor(Pcnt)[1:p, p+c(1:p)]), 
     y=diag(pcor(Pmwt)[1:p, p+c(1:p)]), 
     pch=20, xlab=c("cancer"), ylab=("metastasis"))

# for network visualization: sparsify the left-upper block of the precision matrices
Pcnt0 <- sparsify(Pcnt[1:p, 1:p], threshold="localFDR", FDRcut=0.8)[[1]]
Pmwt0 <- sparsify(Pmwt[1:p, 1:p], threshold="localFDR", FDRcut=0.8)[[1]]

# identify nodes connected in either group.
conNodes <- sort(unique(c(as.numeric(which(adjacentMat(Pcnt0) != 0, arr.ind=TRUE)),
                          as.numeric(which(adjacentMat(Pmwt0) != 0, arr.ind=TRUE)))))

# determine overlap in edges between the groups
table(adjacentMat(Pcnt0)[upper.tri(Pcnt0)], adjacentMat(Pmwt0)[upper.tri(Pmwt0)])

# limit to connected (in either group) nodes
Pcnt0  <- Pcnt0[conNodes, conNodes]
Pmwt0  <- Pmwt0[conNodes, conNodes]

# plot networks of both groups
Ugraph(Pcnt0, type="fancy", lay="layout_with_fr", Vsize=12, Vcex=0.6, 
       Vcol="pink", VLcolor="black", VBcolor="purple", main="Cancer")
Ugraph(Pmwt0, type="fancy", lay="layout_with_fr", Vsize=12, Vcex=0.6, 
       Vcol="lightgreen", VLcolor="black", VBcolor="darkgreen", main="Metastasis")

# set coordinates for common layout
Pall   <- abs(Pcnt0) + abs(Pmwt0)
coords <- Ugraph(Pall , type="fancy", 
                 lay="layout_with_fr", 
                 Vcex=0.7, Vcol="orange", 
                 VLcolor="darkred", VBcolor="yellow")

# plot networks of both groups (with common layout)
Ugraph(Pcnt0, type="fancy", lay=NULL, coords=coords, Vsize=12, Vcex=0.6, 
       Vcol="pink", VLcolor="black", VBcolor="purple", main="Cancer")
Ugraph(Pmwt0, type="fancy", lay=NULL, coords=coords, Vsize=12, Vcex=0.6, 
       Vcol="lightgreen", VLcolor="black", VBcolor="darkgreen", main="Metastasis")

# plot the differential group (with the same layout
DiffGraph(Pcnt0, Pmwt0, lay=NULL, coords=coords, Vsize=12, Vcex=0.6, 
          Vcolor="white", VBcolor="grey", P1color="purple", 
          P2color="green", main="Cancer-Metastasis differential graph")
legend("bottomright", c("cancer", "metastasis"), col=c("purple", "green"), lwd=2)


