# load libraries
library(mvtnorm)
library(glasso)
library(rags2ridges)
library(Matrix)
library(microbenchmark)
library(Rcpp)

# load implementation of generalized ridge and 
# lasso precision estimators 
sourceCpp("generalizedPfunctions.cpp")

# set dimension and sample size
p <- 10
n <- 100

# penalty parameter values
lambda <- 1 * matrix(1, p, p)

# generate banded precision matrix
diags <- list(rep(1, p), rep(0.5, p-1), 
              rep(0.25, p-2), rep(0.1, p-3))
Omega <- as.matrix(bandSparse(p, k = -c(0:3), 
                   diag = c(diags), symm=TRUE))

# undo comment of lines below to generate banded precision matrix
# Omega       <- matrix(0.5, p, p)
# diag(Omega) <- 1

# undo comment of lines below to generate blocked precision matrix
# bSize <- p/5
# Omega <- matrix(0, p, p)
# Omega[c(1:bSize), c(1:bSize)] <- 0.5 
# Omega[1*bSize + c(1:bSize), 1*bSize + c(1:bSize)] <- 0.5 
# Omega[2*bSize + c(1:bSize), 2*bSize + c(1:bSize)] <- 0.5 
# Omega[3*bSize + c(1:bSize), 3*bSize + c(1:bSize)] <- 0.5 
# Omega[4*bSize + c(1:bSize), 4*bSize + c(1:bSize)] <- 0.5 
# diag(Omega) <- 1

# undo comments of lines below to generate hub precision matrix
# Omega       <- matrix(0, p, p)
# Omega[1,]   <- 1/c(1:p)
# Omega[,1]   <- 1/c(1:p)
# diag(Omega) <- 1

# define a zero- target
target <- matrix(0, p, p)


###########################################################
# computing time comparison
###########################################################

# generate data
Y <- rmvnorm(n, sigma=solve(Omega))
S <- covML(Y)

# microbenchmark
ctRes <-microbenchmark(.armaLassoPgen(S, lambda, target,
                       100, 10^(-10), 100, 10^(-10), 10^(5)), 
                       glasso(S, rho=lambda[1,1])$wi)

# administration object
cTimes <- matrix(nrow=100, ncol=2)
colnames(cTimes) <- c("ctGL", "ctLPG")

# summarize results
cTimes[,1] <- ctRes[[2]][substr(levels(ctRes[[1]])[ctRes[[1]]], 
                                1, 14) == ".armaLassoPgen"]
cTimes[,2] <- ctRes[[2]][substr(levels(ctRes[[1]])[ctRes[[1]]], 
                                1, 14) == ".armaLassoPgen"]

# save results
save(cTimes, file=paste("cTimes_lasso_p", p, 
                        "_n",n, ".Rdata", sep=""))


###########################################################
# assessing accuracy
###########################################################

# administration object
accuracy <- matrix(nrow=0, ncol=2)
colnames(accuracy) <- c("accFrob", "accSup")

# evaluate accuracy many times
for (k in 1:100){
	# generate data 
	Y <- rmvnorm(n, sigma=solve(Omega))
	S <- covML(Y)

	# estimate precision with both glasso and 
	# generalized glasso algorithms
	Plasso    <- glasso(S, rho=lambda[1,1])$wi
	PlassoGen <- .armaLassoPgen(S, lambda, target, 100, 10^(-10), 
                                    100, 10^(-10), 10^(5))

	# calculate accuracies
	accuracyFrob <- sqrt(sum((PlassoGen -  Plasso)^2))
	accuracySup  <- max(abs(PlassoGen - Plasso))
	accuracy     <- rbind(accuracy, 
	                      c(accuracyFrob, 
	                      accuracySup))
}

# save results
save(accuracy, file=paste("accLassoos_p", p, 
                          "_n",n, ".Rdata", sep=""))

