# load libraries
library(mvtnorm)
library(rags2ridges)
library(Matrix)
library(Rcpp)

# load implementation of 
# generalized ridge and lasso precision estimators 
sourceCpp("generalizedPfunctions.cpp")

# set dimension and sample size
p <- 10
n <- 100

# penalty parameter values
lambda <- 1 * matrix(1, p, p)

# generate banded precision matrix
diags <- list(rep(1, p), rep(0.5, p-1), rep(0.25, p-2), rep(0.1, p-3))
Omega <- as.matrix(bandSparse(p, k = -c(0:3), diag = c(diags), symm=TRUE))

# define a banded target
diags <- list(rep(0.5, p), rep(0.1, p-1))
target <- as.matrix(bandSparse(p, k = -c(0:1), diag = c(diags), symm=TRUE))


###################################################################################
# loss comparison
###################################################################################

# administration object
accuracy <- matrix(nrow=0, ncol=2)
colnames(accuracy) <- c("accFrobG", "accFrobR")

# evaluate accuracy many times
for (k in 1:100){
	# generate data 
	Y <- rmvnorm(n, sigma=solve(Omega))
	S <- covML(Y)

	# estimate precision with regular ridge
	lambdaOpt <- optPenalty.kCVauto(Y, lambdaMin=0.000001, lambdaMax=100000, lambdaInit=1, target=target)$optLambda
	PridgeR   <- ridgeP(S, lambda=lambdaOpt, target=target)

	# make k-folds as list
	fold         <- 5
	fold         <- max(min(ceiling(fold), nrow(Y)), 2)
	fold         <- rep(1:fold, ceiling(nrow(Y)/fold))[1:nrow(Y)]
	shuffle      <- sample(1:nrow(Y), nrow(Y))
	folds        <- split(shuffle, as.factor(fold))
	names(folds) <- NULL

	# fit for the generalized ridge
	CVs <- numeric()
	lambdas <- exp(seq(log(0.001), log(10), length.out=100))
	for (m in 1:length(lambdas)){	
		# define a banded penalty matrix
		diags <- list(rep(lambdas[m], p))
		for (j in 2:p){
			diags[[j]] <- rep(j*lambdas[m], p-j+1)
		}
		lambdaMat <- as.matrix(bandSparse(p, k = -c(0:(p-1)), diag = c(diags), symm=TRUE))		
		CVs <- c(CVs, .armaKcvlossR(lambdaMat, Y, target, folds, 100, 10^(-10)))	
	}
	lambdaOpt <- lambdas[which.min(CVs)]

	# define a banded penalty matrix
	diags <- list(rep(lambdaOpt, p))
	for (j in 2:p){
		diags[[j]] <- rep(j*lambdaOpt, p-j+1)
	}
	lambdaMat <- as.matrix(bandSparse(p, k = -c(0:(p-1)), diag = c(diags), symm=TRUE))		
	PridgeG   <- .armaRidgePgen(S, lambdaMat, target, nInit=100, minSuccDiff=10^(-10))

	# calculate accuracies
	accFrobG <- sum((PridgeG -  Omega)^2)
	accFrobR <- sum((PridgeR -  Omega)^2)
	accuracy <- rbind(accuracy, c(accFrobG, accFrobR))
}

